print("import: 'networkx'")
import networkx

print("import: 'networkx.algorithms'")
import networkx.algorithms

print("import: 'networkx.algorithms.approximation'")
import networkx.algorithms.approximation

print("import: 'networkx.algorithms.assortativity'")
import networkx.algorithms.assortativity

print("import: 'networkx.algorithms.bipartite'")
import networkx.algorithms.bipartite

print("import: 'networkx.algorithms.centrality'")
import networkx.algorithms.centrality

print("import: 'networkx.algorithms.coloring'")
import networkx.algorithms.coloring

print("import: 'networkx.algorithms.community'")
import networkx.algorithms.community

print("import: 'networkx.algorithms.components'")
import networkx.algorithms.components

print("import: 'networkx.algorithms.connectivity'")
import networkx.algorithms.connectivity

print("import: 'networkx.algorithms.flow'")
import networkx.algorithms.flow

print("import: 'networkx.algorithms.isomorphism'")
import networkx.algorithms.isomorphism

print("import: 'networkx.algorithms.link_analysis'")
import networkx.algorithms.link_analysis

print("import: 'networkx.algorithms.minors'")
import networkx.algorithms.minors

print("import: 'networkx.algorithms.operators'")
import networkx.algorithms.operators

print("import: 'networkx.algorithms.shortest_paths'")
import networkx.algorithms.shortest_paths

print("import: 'networkx.algorithms.traversal'")
import networkx.algorithms.traversal

print("import: 'networkx.algorithms.tree'")
import networkx.algorithms.tree

print("import: 'networkx.classes'")
import networkx.classes

print("import: 'networkx.drawing'")
import networkx.drawing

print("import: 'networkx.generators'")
import networkx.generators

print("import: 'networkx.linalg'")
import networkx.linalg

print("import: 'networkx.readwrite'")
import networkx.readwrite

print("import: 'networkx.readwrite.json_graph'")
import networkx.readwrite.json_graph

print("import: 'networkx.tests'")
import networkx.tests

print("import: 'networkx.utils'")
import networkx.utils

